@@ -8,18 +8,22 @@ import (
 	"net/http"
 	"net/url"
 	"slices"
+	"strings"
 
 	"github.com/gorilla/mux"
 
 	"github.com/lxc/incus/v6/internal/server/auth"
 	"github.com/lxc/incus/v6/internal/server/db"
 	dbCluster "github.com/lxc/incus/v6/internal/server/db/cluster"
+	instanceDrivers "github.com/lxc/incus/v6/internal/server/instance/drivers"
 	"github.com/lxc/incus/v6/internal/server/lifecycle"
 	"github.com/lxc/incus/v6/internal/server/request"
 	"github.com/lxc/incus/v6/internal/server/response"
+	"github.com/lxc/incus/v6/internal/server/state"
 	localUtil "github.com/lxc/incus/v6/internal/server/util"
 	"github.com/lxc/incus/v6/shared/api"
 	"github.com/lxc/incus/v6/shared/logger"
+	"github.com/lxc/incus/v6/shared/osarch"
 	"github.com/lxc/incus/v6/shared/validate"
 )
 
@@ -85,9 +89,20 @@ func clusterGroupsPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Quick checks.
-	err = validate.IsAPIName(req.Name, false)
+	err = clusterGroupValidateName(req.Name)
 	if err != nil {
-		return response.BadRequest(fmt.Errorf("Invalid cluster group name: %w", err))
+		return response.BadRequest(err)
+	}
+
+	err = clusterGroupValidate(req.Config)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Fill in the auto values.
+	err = clusterGroupFill(r.Context(), s, nil, &req.ClusterGroupPut)
+	if err != nil {
+		return response.BadRequest(err)
 	}
 
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -109,6 +124,11 @@ func clusterGroupsPost(d *Daemon, r *http.Request) response.Response {
 			}
 		}
 
+		err = dbCluster.CreateClusterGroupConfig(ctx, tx.Tx(), groupID, req.Config)
+		if err != nil {
+			return err
+		}
+
 		return nil
 	})
 	if err != nil {
@@ -304,11 +324,10 @@ func clusterGroupGet(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(errors.New("This server is not clustered"))
 	}
 
-	var group *dbCluster.ClusterGroup
-
+	var apiGroup *api.ClusterGroup
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get the cluster group.
-		group, err = dbCluster.GetClusterGroup(ctx, tx.Tx(), name)
+		group, err := dbCluster.GetClusterGroup(ctx, tx.Tx(), name)
 		if err != nil {
 			return err
 		}
@@ -323,17 +342,17 @@ func clusterGroupGet(d *Daemon, r *http.Request) response.Response {
 			group.Nodes = append(group.Nodes, node.Node)
 		}
 
+		apiGroup, err = group.ToAPI(ctx, tx.Tx())
+		if err != nil {
+			return err
+		}
+
 		return nil
 	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	apiGroup, err := group.ToAPI()
-	if err != nil {
-		return response.InternalError(err)
-	}
-
 	return response.SyncResponseETag(true, apiGroup, apiGroup.ClusterGroupPut)
 }
 
@@ -387,9 +406,9 @@ func clusterGroupPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Quick checks.
-	err = validate.IsAPIName(req.Name, false)
+	err = clusterGroupValidateName(name)
 	if err != nil {
-		return response.BadRequest(fmt.Errorf("Invalid cluster group name: %w", err))
+		return response.BadRequest(err)
 	}
 
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -467,6 +486,43 @@ func clusterGroupPut(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
+	// Validate the config.
+	err = clusterGroupValidate(req.Config)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Get the current state.
+	var dbClusterGroup *dbCluster.ClusterGroup
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbClusterGroup, err = dbCluster.GetClusterGroup(ctx, tx.Tx(), name)
+		if err != nil {
+			return err
+		}
+
+		nodeClusterGroups, err := dbCluster.GetNodeClusterGroups(ctx, tx.Tx(), dbCluster.NodeClusterGroupFilter{GroupID: &dbClusterGroup.ID})
+		if err != nil {
+			return err
+		}
+
+		dbClusterGroup.Nodes = make([]string, 0, len(nodeClusterGroups))
+		for _, node := range nodeClusterGroups {
+			dbClusterGroup.Nodes = append(dbClusterGroup.Nodes, node.Node)
+		}
+
+		return nil
+	})
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	// Fill in the auto values.
+	err = clusterGroupFill(r.Context(), s, dbClusterGroup.Nodes, &req)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Update the database.
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		group, err := dbCluster.GetClusterGroup(ctx, tx.Tx(), name)
 		if err != nil {
@@ -483,6 +539,11 @@ func clusterGroupPut(d *Daemon, r *http.Request) response.Response {
 			return err
 		}
 
+		err = dbCluster.UpdateClusterGroupConfig(ctx, tx.Tx(), int64(group.ID), req.Config)
+		if err != nil {
+			return err
+		}
+
 		members, err := tx.GetClusterGroupNodes(ctx, name)
 		if err != nil {
 			return err
@@ -598,17 +659,17 @@ func clusterGroupPatch(d *Daemon, r *http.Request) response.Response {
 			dbClusterGroup.Nodes = append(dbClusterGroup.Nodes, node.Node)
 		}
 
+		clusterGroup, err = dbClusterGroup.ToAPI(ctx, tx.Tx())
+		if err != nil {
+			return err
+		}
+
 		return nil
 	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	clusterGroup, err = dbClusterGroup.ToAPI()
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	req := clusterGroup.Writable()
 
 	// Validate the ETag.
@@ -628,6 +689,29 @@ func clusterGroupPatch(d *Daemon, r *http.Request) response.Response {
 		req.Members = clusterGroup.Members
 	}
 
+	if req.Config == nil {
+		req.Config = clusterGroup.Config
+	} else {
+		for k, v := range clusterGroup.Config {
+			_, ok := req.Config[k]
+			if !ok {
+				req.Config[k] = v
+			}
+		}
+	}
+
+	// Validate the config.
+	err = clusterGroupValidate(req.Config)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Fill in the auto values.
+	err = clusterGroupFill(r.Context(), s, dbClusterGroup.Nodes, &req)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		obj := dbCluster.ClusterGroup{
 			Name:        dbClusterGroup.Name,
@@ -644,6 +728,11 @@ func clusterGroupPatch(d *Daemon, r *http.Request) response.Response {
 			return err
 		}
 
+		err = dbCluster.UpdateClusterGroupConfig(ctx, tx.Tx(), groupID, req.Config)
+		if err != nil {
+			return err
+		}
+
 		err = dbCluster.DeleteNodeClusterGroup(ctx, tx.Tx(), int(groupID))
 		if err != nil {
 			return err
@@ -764,3 +853,123 @@ func clusterGroupDelete(d *Daemon, r *http.Request) response.Response {
 
 	return response.EmptySyncResponse
 }
+
+func clusterGroupValidateName(name string) error {
+	if name == "" {
+		return errors.New("No name provided")
+	}
+
+	if strings.Contains(name, "/") {
+		return errors.New("Cluster group names may not contain slashes")
+	}
+
+	if strings.Contains(name, " ") {
+		return errors.New("Cluster group names may not contain spaces")
+	}
+
+	if strings.Contains(name, "_") {
+		return errors.New("Cluster group names may not contain underscores")
+	}
+
+	if strings.Contains(name, "'") || strings.Contains(name, `"`) {
+		return errors.New("Cluster group names may not contain quotes")
+	}
+
+	if name == "*" {
+		return errors.New("Reserved cluster group name")
+	}
+
+	if slices.Contains([]string{".", ".."}, name) {
+		return fmt.Errorf("Invalid cluster group name %q", name)
+	}
+
+	return nil
+}
+
+// clusterGroupValidate validates the configuration keys/values for cluster groups.
+func clusterGroupValidate(config map[string]string) error {
+	configKeys := map[string]func(value string) error{}
+
+	// Add architecture keys.
+	for _, arch := range osarch.SupportedArchitectures() {
+		// gendoc:generate(entity=cluster_group, group=common, key=instances.vm.cpu.ARCHITECTURE.baseline)
+		// The CPU base architecture name as can be found through `qemu -cpu ?`.
+		//
+		// This can be a generic definition like `qemu64` or `kvm64`, or it can be a specific hardware architecture like `EPYC-v2`.
+		// It's important to ensure that all servers in the group match that baseline.
+		// ---
+		//  type: string
+		//  shortdesc: CPU base architecture name
+		configKeys[fmt.Sprintf("instances.vm.cpu.%s.baseline", arch)] = validate.Optional(validate.IsAny)
+
+		// gendoc:generate(entity=cluster_group, group=common, key=instances.vm.cpu.ARCHITECTURE.flags)
+		// A comma separated list of CPU flags to add on top of CPU baseline or a list of flags to remove from it.
+		//
+		// To remove a flag, use `-flag`.
+		// ---
+		//  type: string
+		//  shortdesc: CPU flags to add/remove to/from the baseline
+		configKeys[fmt.Sprintf("instances.vm.cpu.%s.flags", arch)] = validate.Optional(validate.IsListOf(validate.IsAny))
+	}
+
+	for k, v := range config {
+		// User keys are free for all.
+
+		// gendoc:generate(entity=cluster_group, group=common, key=user.*)
+		// User keys can be used in search.
+		// ---
+		//  type: string
+		//  shortdesc: Free form user key/value storage
+		if strings.HasPrefix(k, "user.") {
+			continue
+		}
+
+		validator, ok := configKeys[k]
+		if !ok {
+			return fmt.Errorf("Invalid cluster group configuration key %q", k)
+		}
+
+		err := validator(v)
+		if err != nil {
+			return fmt.Errorf("Invalid cluster group configuration key %q value", k)
+		}
+	}
+
+	return nil
+}
+
+// clusterGroupFill fills in automatic values.
+func clusterGroupFill(ctx context.Context, s *state.State, servers []string, req *api.ClusterGroupPut) error {
+	// If no config, nothing to fill.
+	if req.Config == nil {
+		return nil
+	}
+
+	for _, arch := range osarch.SupportedArchitectures() {
+		baseline := req.Config[fmt.Sprintf("instances.vm.cpu.%s.baseline", arch)]
+		flags := req.Config[fmt.Sprintf("instances.vm.cpu.%s.flags", arch)]
+
+		// Check whether we need to fill in values.
+		if flags != "auto" {
+			continue
+		}
+
+		if baseline != "kvm64" || arch != "x86_64" {
+			return errors.New("Automatic CPU flags are currently only supported on \"x86_64\" with the \"kvm64\" baseline")
+		}
+
+		if len(servers) == 0 {
+			return errors.New("Can't compute automatic CPU flags when no servers are in the cluster group")
+		}
+
+		// Fill in the flags.
+		cpuFlags, err := instanceDrivers.GetClusterCPUFlags(ctx, s, servers, arch)
+		if err != nil {
+			return fmt.Errorf("Couldn't compute automatic CPU flags: %w", err)
+		}
+
+		req.Config[fmt.Sprintf("instances.vm.cpu.%s.flags", arch)] = strings.Join(cpuFlags, ",")
+	}
+
+	return nil
+}
