@@ -299,6 +299,7 @@ func OVNEnsureACLs(s *state.State, l logger.Logger, client *ovn.NB, aclProjectNa
 		}
 
 		// Now apply our ACL rules to port group (and any per-ACL-per-network port groups needed).
+		aclStatus.aclInfo.Project = aclProjectName
 		err = ovnApplyToPortGroup(s, l, client, aclStatus.aclInfo, portGroupName, aclNameIDs, aclNets, peerTargetNetIDs)
 		if err != nil {
 			return nil, fmt.Errorf("Failed applying ACL rules to port group %q for security ACL %q setup: %w", portGroupName, aclStatus.name, err)
@@ -328,7 +329,7 @@ func OVNEnsureACLs(s *state.State, l logger.Logger, client *ovn.NB, aclProjectNa
 		// (and any per-ACL-per-network port groups needed).
 		if aclStatus.aclInfo != nil {
 			l.Debug("Applying ACL rules to OVN port group", logger.Ctx{"networkACL": aclStatus.name, "portGroup": portGroupName})
-
+			aclStatus.aclInfo.Project = aclProjectName
 			err := ovnApplyToPortGroup(s, l, client, aclStatus.aclInfo, portGroupName, aclNameIDs, aclNets, peerTargetNetIDs)
 			if err != nil {
 				return nil, fmt.Errorf("Failed applying ACL rules to port group %q for security ACL %q setup: %w", portGroupName, aclStatus.name, err)
@@ -374,13 +375,72 @@ func ovnAddReferencedACLs(info *api.NetworkACL, referencedACLNames map[string]st
 	}
 }
 
+// replaceAddressSetNames performs replacements of address set names with OVN identifiers.
+func replaceAddressSetNames(subject string, addressSetIDs map[string]int) string {
+	subjects := util.SplitNTrimSpace(subject, ",", -1, true)
+	for i, subj := range subjects {
+		after, ok := strings.CutPrefix(subj, "$")
+		if ok {
+			setID, found := addressSetIDs[after]
+			if found {
+				subjects[i] = fmt.Sprintf("$incus_set%d", setID)
+			}
+		}
+	}
+
+	return strings.Join(subjects, ",")
+}
+
 // ovnApplyToPortGroup applies the rules in the specified ACL to the specified port group.
 func ovnApplyToPortGroup(s *state.State, l logger.Logger, client *ovn.NB, aclInfo *api.NetworkACL, portGroupName ovn.OVNPortGroup, aclNameIDs map[string]int64, aclNets map[string]NetworkACLUsage, peerTargetNetIDs map[cluster.NetworkPeerConnection]int64) error {
 	// Create slice for port group rules that has the capacity for ingress and egress rules, plus default rule.
 	portGroupRules := make([]ovn.OVNACLRule, 0, len(aclInfo.Ingress)+len(aclInfo.Egress)+1)
 	networkRules := make([]ovn.OVNACLRule, 0)
 	networkPeersNeeded := make([]cluster.NetworkPeerConnection, 0)
+	// First gather used address sets
+	addressSetNamesSet := make(map[string]struct{})
 
+	extractAddressSets := func(rules []api.NetworkACLRule) {
+		for _, rule := range rules {
+			for _, subj := range util.SplitNTrimSpace(rule.Source, ",", -1, true) {
+				if strings.HasPrefix(subj, "$") {
+					addressSetNamesSet[subj] = struct{}{}
+				}
+			}
+			for _, subj := range util.SplitNTrimSpace(rule.Destination, ",", -1, true) {
+				if strings.HasPrefix(subj, "$") {
+					addressSetNamesSet[subj] = struct{}{}
+				}
+			}
+		}
+	}
+
+	extractAddressSets(aclInfo.Ingress)
+	extractAddressSets(aclInfo.Egress)
+
+	addressSetNames := make([]string, 0, len(addressSetNamesSet))
+	for setName := range addressSetNamesSet {
+		addressSetNames = append(addressSetNames, setName)
+	}
+
+	// Map address set names to ID
+	addressSetIDs := make(map[string]int, len(addressSetNames))
+	if len(addressSetNames) > 0 {
+		for _, setName := range addressSetNames {
+			err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				set, err := cluster.GetNetworkAddressSet(ctx, tx.Tx(), aclInfo.Project, strings.TrimPrefix(setName, "$"))
+				if err != nil {
+					return err
+				}
+
+				addressSetIDs[set.Name] = set.ID
+				return nil
+			})
+			if err != nil {
+				return fmt.Errorf("Failed fetching address set %s IDs in project %s err: %w", strings.TrimPrefix(setName, "$"), aclInfo.Project, err)
+			}
+		}
+	}
 	// convertACLRules converts the ACL rules to OVN ACL rules.
 	convertACLRules := func(direction string, rules ...api.NetworkACLRule) error {
 		for ruleIndex, rule := range rules {
@@ -388,6 +448,10 @@ func ovnApplyToPortGroup(s *state.State, l logger.Logger, client *ovn.NB, aclInf
 				continue
 			}
 
+			// Replace address set subjects
+			rule.Source = replaceAddressSetNames(rule.Source, addressSetIDs)
+			rule.Destination = replaceAddressSetNames(rule.Destination, addressSetIDs)
+
 			ovnACLRule, networkSpecific, networkPeers, err := ovnRuleCriteriaToOVNACLRule(s, direction, &rule, portGroupName, aclNameIDs, peerTargetNetIDs)
 			if err != nil {
 				return err
@@ -620,7 +684,7 @@ func ovnRuleSubjectToOVNACLMatch(s *state.State, direction string, aclNameIDs ma
 
 				fieldParts = append(fieldParts, fmt.Sprintf("%s.%s == %s", protocol, direction, subjectCriterion))
 			} else {
-				// If not valid IP subnet, check if subject is ACL name or network peer name.
+				// If not valid IP subnet, check if subject is ACL name or address set or network peer name.
 				var subjectPortSelector ovn.OVNPortGroup
 				if slices.Contains(ruleSubjectInternalAliases, subjectCriterion) {
 					// Use pseudo port group name for special reserved port selector types.
@@ -634,6 +698,11 @@ func ovnRuleSubjectToOVNACLMatch(s *state.State, direction string, aclNameIDs ma
 					// Convert deprecated #external to non-deprecated @external if needed.
 					subjectPortSelector = ovn.OVNPortGroup(ruleSubjectExternal)
 					networkSpecific = true
+				} else if strings.HasPrefix(subjectCriterion, "$") {
+					// Check if subject is an address set if so we use it as it is.
+					fieldParts = append(fieldParts, fmt.Sprintf("ip6.%s == %s_ip6 || ip4.%s == %s_ip4", direction, subjectCriterion, direction, subjectCriterion))
+
+					continue
 				} else {
 					after, ok := strings.CutPrefix(subjectCriterion, "@")
 					if ok {
