@@ -21,7 +21,6 @@ import (
 	"github.com/lxc/incus/v6/shared/idmap"
 	"github.com/lxc/incus/v6/shared/logger"
 	"github.com/lxc/incus/v6/shared/osarch"
-	"github.com/lxc/incus/v6/shared/util"
 )
 
 // InotifyTargetInfo records the inotify information associated with a given
@@ -101,7 +100,6 @@ type OS struct {
 	KernelVersion version.DottedVersion
 	Uname         *linux.Utsname
 	BootTime      time.Time
-	IncusOS       bool
 }
 
 // DefaultOS returns a fresh uninitialized OS instance with default values.
@@ -205,8 +203,6 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 		s.KernelVersion = *kernelVersion
 	}
 
-	s.IncusOS = util.PathExists("/var/lib/incus-os/")
-
 	// Fill in the boot time.
 	out, err := os.ReadFile("/proc/stat")
 	if err != nil {
